#!/bin/sh
# $Id: t-mts-acc-10.sh,v 1.2 2007/01/06 04:02:53 ca Exp $
#
# ----------------------------------------
# test whether smtps informs smar about access map usage.
# uses smtpcnf.sh.
# ----------------------------------------
#
test -s stop && exit 1
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

SCRIPTS=`cd ${SD} && pwd`

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || exit 1

# remove map
rm -f ${ACCMAP}

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 9; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 9; nameserver = ${NS}; }
smtpc { Log_Level = 9; remote_port=${SNKPORT}; wait_for_server=4; }

smtps {
Log_Level = 9;
flags = {access};
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
}
EOF

# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
sender:	local:
user:	local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}

${CAT} > ${MT} <<EOF
local.host	lmtp:
other.host	[127.0.0.1]
EOF

# run some test programs

# ----------------------------------------
# access map requested but not available
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
if ${SHELL} ${SD}/smtpcnf.sh -3 lmtpsock -Ssender@local.host -Ruser@local.host -E 0 -X 0 -A 0
then
  if ${GREP} "sev=ERROR, func=smar_react, where=startup, status=access map requested but not available" ${AML} >/dev/null
  then
    :
  else
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
else
   ${VERBOSE} && echo "PASS: ${TEST}"
fi
fi

# ----------------------------------------
# access map requested and available, but client is rejected.
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
# create restrictions
${CAT} > ${ACC} <<EOF
cltaddr:${MTA_LOCALHOST}	quick:reject
EOF
# create map
rm -f ${ACCMAP}
${MM} -t'	' -F ${ACCMAP} < ${ACC}
ERR=false
if ${SHELL} ${SD}/smtpcnf.sh -3 lmtpsock -Ssender@local.host -Ruser@local.host -E 0 -X 0 -A 0
then
  if ${GREP} "ERROR" ${AML} >/dev/null
  then
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
else
   ${VERBOSE} && echo "PASS: ${TEST}"
fi
fi


# ----------------------------------------
rm -f ${ACCMAP} ${ALIMAP} ${MTMAP}

# ----------------------------------------
# end
if test "${ERRS}" = "0"
then
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
