#!/bin/sh
# $Id: t-dns-2.sh,v 1.2 2007/12/01 18:08:09 ca Exp $
# Copyright (c) 2007 Claus Assmann
#
# By using this file, you agree to the terms and conditions set
# forth in the license/LICENSE.3C file which can be found at the
# top level of this source code distribution.
#
# ----------------------------------------
# test handling of "bad" MX records.
# run t-dnstsk-1 with various parameters/arguments
# depends on meta1.org DNS data.
# ----------------------------------------
#
test -s stop && exit 1

if test X"${MTA_NO_DNS_TEST}" != "X"
then
  echo "SKIPPED: $0 {MTA_NO_DNS_TEST} is set"
  exit 0
fi

if test X"${MTA_TEST_DNS_TIMING}" = "X"
then
  echo "SKIPPED: $0 {MTA_TEST_DNS_TIMING} is not set"
  exit 0
fi

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../
fi
GETNS=${SD}/../misc/getns.sh
if test -x ${GETNS}
then
  . ${GETNS}
else
NS=`grep '^nameserver' /etc/resolv.conf | head -n1 | sed -e 's/nameserver[	 ]*\([0-9][0-9\.]*\).*$/\1/'`
fi
if test -z "${NS}"
then
  echo "$0: no nameserver found"
  exit 0
fi

ERRS=0
OPTS="-V"
OUT=t-dns-2.out
OK=t-dns-2.ok
TPRG=./t-dnstsk-1

if ${TPRG} -v | grep "_FFR_CUT_AFTER_INVALID_MX" >/dev/null
then
  FFR_CUT_AFTER_INVALID_MX=true
else
  FFR_CUT_AFTER_INVALID_MX=false
fi

if ${TPRG} -v | grep "_FFR_BE_NICE_TO_INVALID_MX" >/dev/null
then
  FFR_BE_NICE_TO_INVALID_MX=true
else
  FFR_BE_NICE_TO_INVALID_MX=false
fi

if test X"${MTA_DNS_TIMEOUT}" != "X"
then
  OPTS="${OPTS} -O ${MTA_DNS_TIMEOUT}"
fi

# ----------------------------------------
TEST=1
if ${FFR_BE_NICE_TO_INVALID_MX}
then
sort > ${OK} <<EOF
IPv4=A010203
EOF
else
cat > ${OK} <<EOF
EOF
fi

NAME="meta1oknullMX1.meta1.org."
if ${TPRG} ${OPTS} $@ -i ${NS} ${NAME} 2>&1 | egrep 'IPv4|DNSR|CNAME' | sort > ${OUT}
then
  if diff ${OK} ${OUT}
  then
    :
  else
    echo "$0: test ${TEST} failed for ${NAME}"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
  fi
else
  echo "$0: ${TPRG} failed for ${NAME}"
  ERRS=`expr ${ERRS} + 1 `
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

# ----------------------------------------
TEST=2
if ${FFR_BE_NICE_TO_INVALID_MX}
then
if ${FFR_CUT_AFTER_INVALID_MX}
then
cat > ${OK} <<EOF
IPv4=A010203
EOF
else
sort > ${OK} <<EOF
IPv4=A010203
IPv4=A010205
EOF
fi
else
cat > ${OK} <<EOF
EOF
fi

NAME="meta1oknullokMX1.meta1.org."
if ${TPRG} ${OPTS} $@ -i ${NS} ${NAME} 2>&1 | egrep 'IPv4|DNSR|CNAME' | sort > ${OUT}
then
  if diff ${OK} ${OUT}
  then
    :
  else
    echo "$0: test ${TEST} failed for ${NAME}"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
  fi
else
  echo "$0: ${TPRG} failed for ${NAME}"
  ERRS=`expr ${ERRS} + 1 `
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi

# ----------------------------------------
TEST=3
if ${FFR_BE_NICE_TO_INVALID_MX}
then
if ${FFR_CUT_AFTER_INVALID_MX}
then
cat > ${OK} <<EOF
EOF
else
cat > ${OK} <<EOF
IPv4=A010203
EOF
fi
else
cat > ${OK} <<EOF
EOF
fi

NAME="meta1nullMX1.meta1.org."
if ${TPRG} ${OPTS} $@ -i ${NS} ${NAME} 2>&1 | egrep 'IPv4|DNSR|CNAME' | sort > ${OUT}
then
  if diff ${OK} ${OUT}
  then
    :
  else
    echo "$0: test ${TEST} failed for ${NAME}"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
  fi
else
  echo "$0: ${TPRG} failed for ${NAME}"
  ERRS=`expr ${ERRS} + 1 `
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi


rm -f ${OUT} ${OK}

exit ${ERRS}
