#!/bin/sh
# $Id: t-access-6.sh,v 1.7 2008/07/13 02:10:25 ca Exp $
#
# ------------------------------------------------------------
# Description:
# Script to test smtps_session_conf and session_features
# Uses smar, t-access-0, access map, t-mm, startsmar.sh
# ------------------------------------------------------------
# Note: smar and t-access-0 are running asynchronous, hence
# there must be no expected order in the output file.
#
# don't run if stop is "set".
test -s stop && exit 1
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../smar
fi
. ${SD}/common.sh

OUT=access-6.out
IP=access-6.ip
OK=access-6.ok
TPRG=./t-access-0
VERBOSE=false

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

rm -f ${ACCMAP} ${SOCK} ${PIDS}

# create access map for tests
${VALGRIND} ${MM} -t'	' -F ${ACCMAP} <<EOF
smtps_session_conf:127.0.0.1	tls_requirements { cert_issuer="/C=US/ST=California/L=Berkeley/O=Example+20Org/OU=MTA/CN=CA/$EM=MISMATCH@example.org";}
smtps_session_conf:10	tls_requirements { common_name="/C=US/O=MeTA1/"; min_cipher_bits=129;}
smtps_session_conf:1.2.3	tls_requirements { cert_issuer="/C=US/O=MeTA1/"; min_cipher_bits=192;}
session_features:127.0.0.1	localhost
session_features:10	localnet
session_features:1.2.3	onetwothree
cltaddr:1.2.3.10	error:550 5.7.1 reject 1.2.3.10
EOF

killit()
{
if test -s ${PIDS}
then
  # stop SMAR
  for i in `cat ${PIDS}`
  do
    kill ${i}
  done
fi
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi
exit 1
}
trap killit 2


# start SMAR
${SHELL} ${SD}/startsmar.sh || exit 1
rm -f ${OUT} ${IP} ${OK}

TEST=1
# ----------------------------------------
if echo "${SKIP} X" | grep -v ' '${TEST}' ' > /dev/null
then
${VALGRIND} ${TPRG} -S '127.0.0.1' > ${OUT} 2>&1
grep '^react=rhsconf, ' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=rhsconf, ret=0, resconf=0x1
react=rhsconf, ret=0, rhsconf='tls_requirements { cert_issuer="/C=US/ST=California/L=Berkeley/O=Example+20Org/OU=MTA/CN=CA/$EM=MISMATCH@example.org";}'
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi
fi

TEST=2
# ----------------------------------------
if echo "${SKIP} X" | grep -v ' '${TEST}' ' > /dev/null
then
${VALGRIND} ${TPRG} -S '10.1.2.3' > ${OUT} 2>&1
grep '^react=rhsconf, ' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=rhsconf, ret=0, resconf=0x1
react=rhsconf, ret=0, rhsconf='tls_requirements { common_name="/C=US/O=MeTA1/"; min_cipher_bits=129;}'
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi
fi

TEST=3
# ----------------------------------------
if echo "${SKIP} X" | grep -v ' '${TEST}' ' > /dev/null
then
${VALGRIND} ${TPRG} -S '9.1.2.3' > ${OUT} 2>&1
grep '^react=rhsconf, ' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=rhsconf, ret=0, resconf=0x8c040108
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi
fi

TEST=4
# ----------------------------------------
if echo "${SKIP} X" | grep -v ' '${TEST}' ' > /dev/null
then
${VALGRIND} ${TPRG} -S '1.2.3.4' > ${OUT} 2>&1
grep '^react=rhsconf, ' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=rhsconf, ret=0, resconf=0x1
react=rhsconf, ret=0, rhsconf='tls_requirements { cert_issuer="/C=US/O=MeTA1/"; min_cipher_bits=192;}'
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi
fi

# ----------------------------------------
if echo "${SKIP} X" | grep -v ' '${TEST}' ' > /dev/null
then
${VALGRIND} ${TPRG} -f '1.2.3.5' > ${OUT} 2>&1
grep '^react=' ${OUT} | sort > ${IP}
sort > ${OK} <<EOF
react=rvrs_st, ret=0, v=0x2
react=no_match
react=rhsconf, ret=0, resconf=0x1
react=rhsconf, ret=0, rhsconf='tls_requirements { cert_issuer="/C=US/O=MeTA1/"; min_cipher_bits=192;}'
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi
fi

TEST=6
# ----------------------------------------
if echo "${SKIP} X" | grep -v ' '${TEST}' ' > /dev/null
then
${VALGRIND} ${TPRG} -f '1.2.3.10' > ${OUT} 2>&1
grep '^react=' ${OUT} | sort > ${IP}
sort > ${OK} <<EOF
react=statt, ret=0, statt='550 5.7.1 reject 1.2.3.10
react=rvrs_st, ret=0, v=0x2
react=rhsconf, ret=0, resconf=0x1
react=rhsconf, ret=0, rhsconf='tls_requirements { cert_issuer="/C=US/O=MeTA1/"; min_cipher_bits=192;}'
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
  test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
fi
fi

# TEST=7
# # ----------------------------------------
# ${VALGRIND} ${TPRG} -fs '1.2.3.10' > ${OUT} 2>&1
# grep '^react=' ${OUT} | sort > ${IP}
# sort > ${OK} <<EOF
# react=statt, ret=0, statt='550 5.7.1 reject 1.2.3.10
# react=rvrs_st, ret=0, v=0x2
# react=rhs2, ret=0, res2=0
# react=rhs2, ret=0, rhs2='onetwothree'
# react=rhsconf, ret=0, resconf=0x1
# react=rhsconf, ret=0, rhsconf='tls_requirements { cert_issuer="/C=US/O=MeTA1/"; min_cipher_bits=192;}'
# EOF
# if diff ${OK} ${IP}
# then
#   :
# else
#   ERRS=`expr ${ERRS} + 1 `
#   echo "$0: test ${TEST} failed."
#   test X"${MTA_STOPONERROR}" != X && exit ${ERRS}
# fi


# ----------------
# stop SMAR
for i in `cat ${PIDS}`
do
  kill ${i}
done
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi
sleep 3

exit ${ERRS}
